@@ -148,6 +148,7 @@ impl Screen<'_> {
         let sugarloaf_renderer = SugarloafRenderer {
             power_preference,
             backend,
+            font_features: config.fonts.features.clone(),
         };
 
         let mut sugarloaf: Sugarloaf = match Sugarloaf::new(
@@ -193,7 +194,6 @@ impl Screen<'_> {
             working_dir,
             spawn_performer: true,
             use_fork: config.use_fork,
-            is_collapsed,
             is_native,
             // When navigation is collapsed and does not contain any color rule
             // does not make sense fetch for foreground process names
@@ -204,6 +204,7 @@ impl Screen<'_> {
             (&state.get_cursor_state(), config.blinking_cursor),
             event_proxy,
             window_id,
+            0,
             context_manager_config,
             sugarloaf.layout_next(),
             sugarloaf_errors,
@@ -294,13 +295,11 @@ impl Screen<'_> {
         current_theme: Option<winit::window::Theme>,
         font_library: &rio_backend::sugarloaf::font::FontLibrary,
     ) {
-        self.sugarloaf.update_font(font_library);
-
         let padding_y_bottom = padding_bottom_from_config(config);
-
         let padding_y_top = padding_top_from_config(config);
 
-        self.sugarloaf.layout_next().recalculate(
+        self.sugarloaf.update_font(font_library);
+        self.sugarloaf.layout_next_mut().recalculate(
             config.fonts.size,
             config.line_height,
             config.padding_x,
@@ -308,7 +307,7 @@ impl Screen<'_> {
             padding_y_bottom,
         );
 
-        self.sugarloaf.layout_next().update();
+        self.sugarloaf.layout_next_mut().update();
         self.state = State::new(config, current_theme);
 
         for context in self.ctx().contexts() {
@@ -328,17 +327,6 @@ impl Screen<'_> {
             self.sugarloaf.set_background_image(image);
         }
 
-        // TODO: Forcefull hack to compute render updates
-        // Currently sugarloaf, doesn't render if dimensions has changed
-        // and it returns a flag dimensions_changed as true
-        // Ideally resize_all_contexts should be passed as a pointer so we can trigger
-        // in the sugarloaf size and then render there instead.
-        self.render();
-
-        if self.sugarloaf.dimensions_changed() {
-            self.resize_all_contexts();
-        };
-
         self.render();
         self.resize_all_contexts();
     }
@@ -353,17 +341,6 @@ impl Screen<'_> {
 
         self.sugarloaf.update_font_size(action);
 
-        // TODO: Forcefull hack to compute render updates
-        // Currently sugarloaf, doesn't render if dimensions has changed
-        // and it returns a flag dimensions_changed as true
-        // Ideally resize_all_contexts should be passed as a pointer so we can trigger
-        // in the sugarloaf size and then render there instead.
-        self.render();
-
-        if self.sugarloaf.dimensions_changed() {
-            self.resize_all_contexts();
-        };
-
         self.render();
         self.resize_all_contexts();
     }
@@ -384,10 +361,7 @@ impl Screen<'_> {
         self.sugarloaf.rescale(new_scale);
         self.sugarloaf.resize(new_size.width, new_size.height);
         self.render();
-
-        if self.sugarloaf.dimensions_changed() {
-            self.resize_all_contexts();
-        }
+        self.resize_all_contexts();
 
         self
     }
@@ -518,7 +492,11 @@ impl Screen<'_> {
                 // preventing them from being used in bindings
                 //
                 // For more see https://github.com/rust-windowing/winit/issues/2945.
-                if (cfg!(windows) && mods.control_key()) && mods.alt_key() {
+                // if (cfg!(target_os = "macos") || (cfg!(windows) && mods.control_key()))
+                //     && mods.alt_key()
+                if (mods.shift_key() || mods.alt_key())
+                    || mods.alt_key() && (cfg!(windows) && mods.control_key())
+                {
                     key.key_without_modifiers()
                 } else {
                     Key::Character(ch.to_lowercase().into())
@@ -747,9 +725,11 @@ impl Screen<'_> {
                     }
                     Act::SelectTab(tab_index) => {
                         self.context_manager.select_tab(*tab_index);
+                        self.render();
                     }
                     Act::SelectLastTab => {
                         self.context_manager.select_last_tab();
+                        self.render();
                     }
                     Act::SelectNextTab => {
                         self.clear_selection();
@@ -1217,6 +1197,10 @@ impl Screen<'_> {
     pub fn render(&mut self) {
         // If sugarloaf does have pending updates to process then
         // should abort current render
+
+        // let start = std::time::Instant::now();
+        // println!("Render time elapsed");
+
         #[cfg(use_wa)]
         if self.sugarloaf.dimensions_changed() {
             self.resize_all_contexts();
@@ -1253,6 +1237,9 @@ impl Screen<'_> {
         {
             self.context_manager.schedule_render(800);
         }
+
+        // let duration = start.elapsed();
+        // println!("Total render time is: {:?}\n", duration);
     }
 
     fn sgr_mouse_report(&mut self, pos: Pos, button: u8, state: ElementState) {
