@@ -79,7 +79,7 @@ impl Sequencer {
                                 if let Some(route) =
                                     self.router.routes.get_mut(&window_id)
                                 {
-                                    route.redraw();
+                                    route.window.winit_window.request_redraw();
                                 }
                             }
                             RioEventType::Rio(RioEvent::Render) => {
@@ -91,7 +91,18 @@ impl Sequencer {
                                     {
                                         return;
                                     }
-                                    route.redraw();
+                                    route.window.winit_window.request_redraw();
+                                }
+                            }
+                            RioEventType::Rio(RioEvent::RenderRoute(route_id)) => {
+                                if let Some(route) =
+                                    self.router.routes.get_mut(&window_id)
+                                {
+                                    if route_id
+                                        == route.window.screen.ctx().current_route()
+                                    {
+                                        route.window.winit_window.request_redraw();
+                                    }
                                 }
                             }
                             RioEventType::Rio(RioEvent::UpdateGraphicLibrary) => {
@@ -231,6 +242,25 @@ impl Sequencer {
                                     );
                                 }
                             }
+                            RioEventType::Rio(RioEvent::PrepareRenderOnRoute(
+                                millis,
+                                route_id,
+                            )) => {
+                                let timer_id = TimerId::new(Topic::Render, 0);
+                                let event = EventPayload::new(
+                                    RioEventType::Rio(RioEvent::RenderRoute(route_id)),
+                                    window_id,
+                                );
+
+                                if !scheduler.scheduled(timer_id) {
+                                    scheduler.schedule(
+                                        event,
+                                        Duration::from_millis(millis),
+                                        false,
+                                        timer_id,
+                                    );
+                                }
+                            }
                             RioEventType::Rio(RioEvent::Title(title)) => {
                                 if let Some(route) =
                                     self.router.routes.get_mut(&window_id)
@@ -332,7 +362,7 @@ impl Sequencer {
                                 // Rio doesn't cover this case yet.
                                 //
                                 // In the future should try first get
-                                // from Crosrouteords then state colors
+                                // from Crosswords then state colors
                                 // screen.colors()[index] or screen.state.colors[index]
                                 if let Some(route) =
                                     self.router.routes.get_mut(&window_id)
@@ -361,8 +391,6 @@ impl Sequencer {
                                 working_dir_overwrite,
                             )) => {
                                 if let Some(route) = self.router.routes.get(&window_id) {
-                                    route.redraw();
-
                                     // This case happens only for native tabs
                                     // every time that a new tab is created through context
                                     // it also reaches for the foreground process path if
@@ -1014,8 +1042,6 @@ impl Sequencer {
                                 && self.config.hide_cursor_when_typing
                             {
                                 route.window.winit_window.set_cursor_visible(false);
-
-                                // route.redraw();
                             }
                         }
                     }
