@@ -10,11 +10,12 @@ use crate::context::Context;
 use crate::font::FontLibraryData;
 use crate::layout::SugarDimensions;
 use compositor::{
-    CachedRect, Command, Compositor, DisplayList, Rect, TextureEvent, TextureId, Vertex,
+    CachedRun, CachedRunGlyph, Command, Compositor, DisplayList, Rect, TextureEvent,
+    TextureId, Vertex,
 };
 use rustc_hash::FxHashMap;
 use std::{borrow::Cow, mem};
-use text::{Glyph, TextRunStyle, UnderlineStyle};
+use text::{Glyph, TextRunStyle};
 use wgpu::util::DeviceExt;
 use wgpu::Texture;
 
@@ -320,14 +321,15 @@ impl RichTextBrush {
 
         draw_layout(
             &mut self.comp,
-            &mut self.images,
-            &mut self.glyphs,
+            (
+                &mut self.images,
+                &mut self.glyphs,
+                &mut self.draw_layout_cache,
+            ),
             &state.compositors.advanced.render_data,
-            state.current.layout.style.screen_position.0,
-            state.current.layout.style.screen_position.1,
+            state.current.layout.style.screen_position,
             font_library,
             state.current.layout.dimensions,
-            &mut self.draw_layout_cache,
         );
         self.draw_layout_cache.clear_on_demand();
         // let duration = start.elapsed();
@@ -352,8 +354,7 @@ impl RichTextBrush {
 
         let dimension = fetch_dimensions(
             &mut self.comp,
-            &mut self.images,
-            &mut self.glyphs,
+            (&mut self.images, &mut self.glyphs),
             &state.compositors.advanced.mocked_render_data,
             font_library,
         );
@@ -678,18 +679,18 @@ impl RichTextBrush {
 
 #[derive(Default)]
 struct DrawLayoutCache {
-    inner: FxHashMap<String, Vec<CachedRect>>,
+    inner: FxHashMap<u64, Vec<CachedRun>>,
 }
 
 impl DrawLayoutCache {
     #[inline]
-    fn get(&self, id: &str) -> Option<&Vec<CachedRect>> {
-        self.inner.get(id)
+    fn get(&self, hash: &u64) -> Option<&Vec<CachedRun>> {
+        self.inner.get(hash)
     }
 
     #[inline]
-    fn insert(&mut self, id: String, data: Vec<CachedRect>) {
-        self.inner.insert(id, data);
+    fn insert(&mut self, hash: u64, data: Vec<CachedRun>) {
+        self.inner.insert(hash, data);
     }
 
     #[inline]
@@ -706,19 +707,17 @@ impl DrawLayoutCache {
 }
 
 #[inline]
-#[allow(clippy::too_many_arguments)]
 fn draw_layout(
     comp: &mut compositor::Compositor,
-    images: &mut ImageCache,
-    glyphs_cache: &mut GlyphCache,
+    caches: (&mut ImageCache, &mut GlyphCache, &mut DrawLayoutCache),
     render_data: &crate::layout::RenderData,
-    x: f32,
-    y: f32,
+    pos: (f32, f32),
     font_library: &FontLibraryData,
     rect: SugarDimensions,
-    draw_layout_cache: &mut DrawLayoutCache,
 ) {
     // let start = std::time::Instant::now();
+    let (x, y) = pos;
+    let (image_cache, glyphs_cache, draw_layout_cache) = caches;
     let depth = 0.0;
     let mut glyphs = Vec::new();
     let mut current_font = 0;
@@ -733,7 +732,7 @@ fn draw_layout(
     }
 
     let mut session = glyphs_cache.session(
-        images,
+        image_cache,
         font_library[current_font].as_ref(),
         &current_font_coords,
         current_font_size,
@@ -742,29 +741,33 @@ fn draw_layout(
     for line in render_data.lines() {
         let hash = line.hash().unwrap_or(0);
         let mut px = x + line.offset();
-        let id = format!("{}-{}-{}", hash, line.baseline(), px);
+        let py = line.baseline() + y;
         if hash > 0 {
-            if let Some(data) = draw_layout_cache.get(&id) {
-                comp.draw_glyphs_from_cache(data, depth);
+            if let Some(data) = draw_layout_cache.get(&hash) {
+                comp.draw_cached_run(data, px, py, depth, rect, line);
                 continue;
             }
         }
 
-        let mut cache = Vec::default();
+        let mut cached_line_runs = Vec::new();
         for run in line.runs() {
+            let char_width = run.char_width();
+            let mut cached_run = CachedRun::new(char_width);
             let font = *run.font();
             let char_width = run.char_width();
 
-            let py = line.baseline() + y;
             let run_x = px;
             glyphs.clear();
             for cluster in run.visual_clusters() {
                 for glyph in cluster.glyphs() {
+                    cached_run
+                        .glyphs
+                        .push(CachedRunGlyph { id: glyph.id, x, y });
+
                     let x = px + glyph.x;
                     let y = py - glyph.y;
-                    // px += glyph.advance;
+                    // px += glyph.advance
                     px += rect.width * char_width;
-                    // cache.glyphs.push(Glyph { id: glyph.id, x, y });
                     glyphs.push(Glyph { id: glyph.id, x, y });
                 }
             }
@@ -781,15 +784,8 @@ fn draw_layout(
                 topline: py - line.ascent(),
                 line_height,
                 advance: px - run_x,
-                underline: if run.underline() {
-                    Some(UnderlineStyle {
-                        offset: run.underline_offset(),
-                        size: run.underline_size(),
-                        color: run.underline_color(),
-                    })
-                } else {
-                    None
-                },
+                decoration: run.decoration(),
+                decoration_color: run.decoration_color(),
             };
 
             if font != current_font
@@ -797,7 +793,7 @@ fn draw_layout(
                 || style.font_coords != current_font_coords
             {
                 session = glyphs_cache.session(
-                    images,
+                    image_cache,
                     style.font,
                     style.font_coords,
                     style.font_size,
@@ -809,18 +805,21 @@ fn draw_layout(
             }
 
             if hash > 0 {
-                cache.extend(comp.draw_glyphs(
+                comp.draw_run(
                     &mut session,
                     Rect::new(run_x, py, style.advance, 1.),
                     depth,
                     &style,
                     glyphs.iter(),
-                ));
+                    &mut cached_run,
+                );
+
+                cached_line_runs.push(cached_run);
             }
         }
 
-        if hash > 0 && !cache.is_empty() {
-            draw_layout_cache.insert(id, cache);
+        if hash > 0 && !cached_line_runs.is_empty() {
+            draw_layout_cache.insert(hash, cached_line_runs);
         }
     }
 
@@ -831,14 +830,14 @@ fn draw_layout(
 #[inline]
 fn fetch_dimensions(
     comp: &mut compositor::Compositor,
-    images: &mut ImageCache,
-    glyphs_cache: &mut GlyphCache,
+    caches: (&mut ImageCache, &mut GlyphCache),
     render_data: &crate::layout::RenderData,
     font_library: &FontLibraryData,
 ) -> SugarDimensions {
     let x = 0.;
     let y = 0.;
 
+    let (image_cache, glyphs_cache) = caches;
     let mut current_font = 0;
     let mut current_font_size = 0.0;
     let mut current_font_coords: Vec<i16> = Vec::with_capacity(0);
@@ -851,7 +850,7 @@ fn fetch_dimensions(
     }
 
     let mut session = glyphs_cache.session(
-        images,
+        image_cache,
         font_library[current_font].as_ref(),
         &current_font_coords,
         current_font_size,
@@ -862,6 +861,9 @@ fn fetch_dimensions(
     for line in render_data.lines() {
         let mut px = x + line.offset();
         for run in line.runs() {
+            let char_width = run.char_width();
+            let mut cached_run = CachedRun::new(char_width);
+
             let font = run.font();
             let py = line.baseline() + y;
             let run_x = px;
@@ -871,7 +873,7 @@ fn fetch_dimensions(
                 for glyph in cluster.glyphs() {
                     let x = px + glyph.x;
                     let y = py - glyph.y;
-                    px += glyph.advance;
+                    px += glyph.advance * char_width;
                     glyphs.push(Glyph { id: glyph.id, x, y });
                 }
             }
@@ -888,7 +890,8 @@ fn fetch_dimensions(
                 topline: py - line.ascent(),
                 line_height,
                 advance: px - run_x,
-                underline: None,
+                decoration: None,
+                decoration_color: None,
             };
 
             if style.advance > 0. && line_height > 0. {
@@ -901,7 +904,7 @@ fn fetch_dimensions(
                 || style.font_coords != current_font_coords
             {
                 session = glyphs_cache.session(
-                    images,
+                    image_cache,
                     style.font,
                     style.font_coords,
                     style.font_size,
@@ -912,12 +915,13 @@ fn fetch_dimensions(
                 current_font_size = style.font_size;
             }
 
-            comp.draw_glyphs(
+            comp.draw_run(
                 &mut session,
                 Rect::new(run_x, py, style.advance, 1.),
                 0.0,
                 &style,
                 glyphs.iter(),
+                &mut cached_run,
             );
         }
     }
