@@ -5,14 +5,12 @@ pub mod fonts;
 pub mod loader;
 
 pub const FONT_ID_REGULAR: usize = 0;
-pub const FONT_ID_ITALIC: usize = 1;
-pub const FONT_ID_BOLD: usize = 2;
-pub const FONT_ID_BOLD_ITALIC: usize = 3;
 
 use crate::font::constants::*;
+use crate::layout::FragmentStyle;
 use crate::SugarloafErrors;
 use ab_glyph::FontArc;
-use std::collections::HashMap;
+use rustc_hash::FxHashMap;
 use std::ops::{Index, IndexMut};
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
@@ -41,38 +39,57 @@ impl Inner {
 
 #[derive(Default)]
 pub struct FontContext {
-    cache: HashMap<String, usize>,
+    cache: FxHashMap<String, usize>,
 }
 
-impl FontContext {
-    #[inline]
-    pub fn lookup_for_font_match(
-        &mut self,
-        cluster: &mut CharCluster,
-        synth: &mut Synthesis,
-        library: &FontLibraryData,
-    ) -> Option<usize> {
-        let mut font_id = None;
-        for (current_font_id, font) in library.inner.iter().enumerate() {
-            let (font, font_ref) = match font {
-                FontSource::Data(font_data) => (font_data, font_data.as_ref()),
-                FontSource::Extension(_) => {
-                    (&library.standard, library.standard.as_ref())
-                }
-                FontSource::Standard => (&library.standard, library.standard.as_ref()),
-            };
-            let charmap = font.charmap_proxy().materialize(&font_ref);
-            let status = cluster.map(|ch| charmap.map(ch));
-            if status != Status::Discard {
-                *synth = library[current_font_id].synth;
-                font_id = Some(current_font_id);
-                break;
+pub fn lookup_for_font_match(
+    cluster: &mut CharCluster,
+    synth: &mut Synthesis,
+    library: &FontLibraryData,
+    spec_font_attr_opt: Option<&(swash::Style, bool)>,
+) -> Option<usize> {
+    let mut font_id = None;
+    for (current_font_id, font) in library.inner.iter().enumerate() {
+        let (font, font_ref) = match font {
+            FontSource::Data(font_data) => (font_data, font_data.as_ref()),
+            FontSource::Extension(_) => (&library.standard, library.standard.as_ref()),
+            FontSource::Standard => (&library.standard, library.standard.as_ref()),
+        };
+
+        // In this case, the font does match however
+        // we need to check if is indeed a match
+        if let Some(spec_font_attr) = spec_font_attr_opt {
+            if font.style != spec_font_attr.0 {
+                continue;
+            }
+
+            // In case bold is required
+            // It follows spec on Bold (>=700)
+            // https://developer.mozilla.org/en-US/docs/Web/CSS/@font-face/font-weight
+            if spec_font_attr.1 && font.weight < swash::Weight(700) {
+                continue;
             }
         }
 
-        font_id
+        let charmap = font.charmap_proxy().materialize(&font_ref);
+        let status = cluster.map(|ch| charmap.map(ch));
+        if status != Status::Discard {
+            *synth = library[current_font_id].synth;
+            font_id = Some(current_font_id);
+            break;
+        }
     }
 
+    // In case no font_id is found and exists a font spec requirement
+    // then drop requirement and try to find something that can match.
+    if font_id.is_none() && spec_font_attr_opt.is_some() {
+        return lookup_for_font_match(cluster, synth, library, None);
+    }
+
+    font_id
+}
+
+impl FontContext {
     #[inline]
     pub fn map_cluster(
         &mut self,
@@ -80,12 +97,32 @@ impl FontContext {
         synth: &mut Synthesis,
         library: &FontLibraryData,
         fonts_to_load: &mut Vec<(usize, PathBuf)>,
+        fragment_style: &FragmentStyle,
     ) -> Option<usize> {
+        let is_italic = fragment_style.font_attrs.2 == Style::Italic;
+        let is_bold = fragment_style.font_attrs.1 == Weight::BOLD;
+
+        let chars = cluster.chars();
+        let is_cache_key_empty = chars.is_empty();
+
         let mut cache_key: String = String::default();
-        for c in cluster.chars().iter() {
+        let spec_font_attr = if is_bold && is_italic {
+            cache_key.push('m');
+            Some((Style::Italic, true))
+        } else if is_bold {
+            cache_key.push('b');
+            Some((Style::Normal, true))
+        } else if is_italic {
+            cache_key.push('i');
+            Some((Style::Italic, false))
+        } else {
+            cache_key.push('r');
+            None
+        };
+
+        for c in chars.iter() {
             cache_key.push(c.ch);
         }
-        let is_cache_key_empty = cache_key.is_empty();
 
         if !is_cache_key_empty {
             if let Some(cached_font_id) = self.cache.get(&cache_key) {
@@ -102,10 +139,13 @@ impl FontContext {
             }
         }
 
-        if let Some(found_font_id) = self.lookup_for_font_match(cluster, synth, library) {
+        if let Some(found_font_id) =
+            lookup_for_font_match(cluster, synth, library, spec_font_attr.as_ref())
+        {
             if !is_cache_key_empty {
                 self.cache.insert(cache_key, found_font_id);
             }
+
             return Some(found_font_id);
         }
 
@@ -142,7 +182,7 @@ impl FontContext {
             }
         }
 
-        None
+        Some(0)
     }
 }
 
@@ -450,8 +490,9 @@ pub struct FontData {
 
 impl PartialEq for FontData {
     fn eq(&self, other: &Self) -> bool {
-        // self.data == other.data && self.offset == other.offset &&
+        // self.data == other.data &&
         self.key == other.key
+        // self.offset == other.offset && self.key == other.key
     }
 }
 
