@@ -15,6 +15,7 @@ pub struct Sugar {
     pub repeated: usize,
     pub foreground_color: [f32; 4],
     pub background_color: Option<[f32; 4]>,
+    pub decoration_color: Option<[f32; 4]>,
     pub style: SugarStyle,
     pub decoration: SugarDecoration,
     pub cursor: SugarCursor,
@@ -37,6 +38,7 @@ impl Default for Sugar {
             repeated: 0,
             foreground_color: [0., 0., 0., 0.],
             background_color: None,
+            decoration_color: None,
             style: SugarStyle::default(),
             decoration: SugarDecoration::default(),
             cursor: SugarCursor::default(),
@@ -61,31 +63,16 @@ impl Hash for Sugar {
             bg_color[2].to_bits().hash(state);
             bg_color[3].to_bits().hash(state);
         }
-        match self.style {
-            SugarStyle::Disabled => {
-                0.hash(state);
-            }
-            SugarStyle::Italic => {
-                1.hash(state);
-            }
-            SugarStyle::Bold => {
-                2.hash(state);
-            }
-            SugarStyle::BoldItalic => {
-                3.hash(state);
-            }
-        };
-        match self.decoration {
-            SugarDecoration::Disabled => {
-                0.hash(state);
-            }
-            SugarDecoration::Underline => {
-                1.hash(state);
-            }
-            SugarDecoration::Strikethrough => {
-                2.hash(state);
-            }
-        };
+
+        if let Some(color) = self.decoration_color {
+            color[0].to_bits().hash(state);
+            color[1].to_bits().hash(state);
+            color[2].to_bits().hash(state);
+            color[3].to_bits().hash(state);
+        }
+
+        (self.style as u8).hash(state);
+        (self.decoration as u8).hash(state);
         match self.cursor {
             SugarCursor::Disabled => {
                 0.hash(state);
@@ -147,20 +134,26 @@ pub enum SugarCursor {
 }
 
 #[derive(Debug, Copy, PartialEq, Default, Clone)]
+#[repr(u8)]
 pub enum SugarDecoration {
-    Underline,
-    Strikethrough,
+    DottedUnderline = 6,
+    DashedUnderline = 5,
+    DoubleUnderline = 4,
+    CurlyUnderline = 3,
+    Strikethrough = 2,
+    Underline = 1,
     #[default]
-    Disabled,
+    Disabled = 0,
 }
 
 #[derive(Debug, PartialEq, Default, Copy, Clone)]
+#[repr(u8)]
 pub enum SugarStyle {
+    BoldItalic = 3,
+    Italic = 2,
+    Bold = 1,
     #[default]
-    Disabled,
-    Italic,
-    Bold,
-    BoldItalic,
+    Disabled = 0,
 }
 
 #[derive(Copy, PartialEq, Default, Debug, Clone)]
@@ -595,6 +588,7 @@ pub mod test {
             repeated: 0,
             foreground_color: [0., 0., 0., 0.],
             background_color: None,
+            decoration_color: None,
             style: SugarStyle::Disabled,
             decoration: SugarDecoration::Disabled,
             cursor: SugarCursor::Disabled,
@@ -608,6 +602,7 @@ pub mod test {
             repeated: 0,
             foreground_color: [0., 0., 0., 0.],
             background_color: None,
+            decoration_color: None,
             style: SugarStyle::Bold,
             decoration: SugarDecoration::Disabled,
             cursor: SugarCursor::Disabled,
@@ -621,6 +616,7 @@ pub mod test {
             repeated: 0,
             foreground_color: [0., 0., 0., 0.],
             background_color: None,
+            decoration_color: None,
             style: SugarStyle::Disabled,
             decoration: SugarDecoration::Strikethrough,
             cursor: SugarCursor::Disabled,
