@@ -14,7 +14,6 @@ pub struct SugarState {
     pub current: Box<SugarTree>,
     pub next: SugarTree,
     latest_change: SugarTreeDiff,
-    dimensions_changed: bool,
     current_line: usize,
     pub is_dirty: bool,
     pub compositors: SugarCompositors,
@@ -26,22 +25,26 @@ impl SugarState {
     pub fn new(
         initial_layout: SugarloafLayout,
         font_library: &FontLibrary,
+        font_features: &Option<Vec<String>>,
     ) -> SugarState {
         // First time computing changes should obtain dimensions
         let next = SugarTree {
             layout: initial_layout,
             ..Default::default()
         };
-        SugarState {
+
+        let mut state = SugarState {
             is_dirty: false,
             current_line: 0,
             compositors: SugarCompositors::new(font_library),
             graphics: SugarloafGraphics::default(),
             current: Box::<SugarTree>::default(),
             next,
-            dimensions_changed: false,
             latest_change: SugarTreeDiff::LayoutIsDifferent,
-        }
+        };
+
+        state.compositors.advanced.set_font_features(font_features);
+        state
     }
 
     #[inline]
@@ -90,23 +93,6 @@ impl SugarState {
         self.next.lines[self.current_line].insert(sugar);
     }
 
-    #[inline]
-    pub fn insert_on_current_line_from_vec(&mut self, sugar_vec: &Vec<&crate::Sugar>) {
-        for sugar in sugar_vec {
-            self.next.lines[self.current_line].insert(sugar);
-        }
-    }
-
-    #[inline]
-    pub fn insert_on_current_line_from_vec_owned(
-        &mut self,
-        sugar_vec: &Vec<crate::Sugar>,
-    ) {
-        for sugar in sugar_vec {
-            self.next.lines[self.current_line].insert(sugar);
-        }
-    }
-
     #[inline]
     pub fn set_fonts(&mut self, fonts: &FontLibrary) {
         self.compositors.advanced.set_fonts(fonts);
@@ -114,6 +100,11 @@ impl SugarState {
         self.next.layout.dimensions.width = 0.0;
     }
 
+    #[inline]
+    pub fn set_font_features(&mut self, font_features: &Option<Vec<String>>) {
+        self.compositors.advanced.set_font_features(font_features);
+    }
+
     #[inline]
     pub fn clean_screen(&mut self) {
         self.current.lines.clear();
@@ -129,14 +120,13 @@ impl SugarState {
 
     #[inline]
     pub fn reset_compositor(&mut self) {
-        self.compositors.elementary.reset();
-        self.dimensions_changed = false;
+        self.compositors.elementary.clean();
+        self.compositors.advanced.reset();
     }
 
     #[inline]
     pub fn clean_compositor(&mut self) {
         self.compositors.elementary.clean();
-        self.dimensions_changed = false;
     }
 
     #[inline]
@@ -147,6 +137,7 @@ impl SugarState {
         rect_brush: &mut RectBrush,
         context: &mut super::Context,
     ) -> bool {
+        #[cfg(not(feature = "always_dirty"))]
         if !self.is_dirty && self.latest_change == SugarTreeDiff::Equal {
             self.compositors.advanced.clean();
             return false;
@@ -154,10 +145,6 @@ impl SugarState {
 
         advance_brush.prepare(context, self);
 
-        for section in &self.compositors.elementary.blocks_sections {
-            elementary_brush.queue(section);
-        }
-
         if self.compositors.elementary.should_resize {
             rect_brush.resize(context);
         }
@@ -167,48 +154,36 @@ impl SugarState {
         // ...
         // If current tree has blocks and compositor has empty blocks
         // It means that's either the first render or blocks were erased on compute_diff() step
-        if !self.current.blocks.is_empty()
-            && self.compositors.elementary.blocks_are_empty()
-        {
-            for block in &self.current.blocks {
-                if let Some(text) = &block.text {
-                    elementary_brush.queue(
-                        self.compositors
-                            .elementary
-                            .create_section_from_text(text, &self.current),
-                    );
-                }
+        for block in &self.current.blocks {
+            if let Some(text) = &block.text {
+                elementary_brush.queue(
+                    &self
+                        .compositors
+                        .elementary
+                        .create_section_from_text(text, &self.current),
+                );
+            }
 
-                if !block.rects.is_empty() {
-                    self.compositors.elementary.extend_block_rects(&block.rects);
-                }
+            if !block.rects.is_empty() {
+                self.compositors.elementary.rects.extend(&block.rects);
             }
         }
 
-        // Add block rects to main rects
-        self.compositors
-            .elementary
-            .rects
-            .extend(&self.compositors.elementary.blocks_rects);
-
         true
     }
 
-    #[inline]
-    pub fn layout_was_updated(&self) -> bool {
-        self.latest_change == SugarTreeDiff::LayoutIsDifferent
-    }
-
     #[inline]
     pub fn compute_dimensions(&mut self, advance_brush: &mut RichTextBrush) {
         // If layout is different or current has empty dimensions
         // then current will flip with next and will try to obtain
         // the dimensions.
 
-        if !self.layout_was_updated() {
+        if self.latest_change != SugarTreeDiff::LayoutIsDifferent {
             return;
         }
 
+        advance_brush.clean_cache();
+
         if let Some(dimension) = advance_brush.dimensions(self) {
             let mut dimensions_changed = false;
             if dimension.height != self.current.layout.dimensions.height {
@@ -227,17 +202,12 @@ impl SugarState {
             if dimensions_changed {
                 self.current.layout.update();
                 self.next.layout = self.current.layout;
-                self.dimensions_changed = true;
+                self.is_dirty = true;
                 log::info!("sugar_state: dimensions has changed");
             }
         }
     }
 
-    #[inline]
-    pub fn dimensions_changed(&self) -> bool {
-        self.dimensions_changed
-    }
-
     #[inline]
     pub fn reset_next(&mut self) {
         self.next.layout = self.current.layout;
@@ -250,28 +220,32 @@ impl SugarState {
     #[inline]
     pub fn compute_changes(&mut self) {
         // If sugar dimensions are empty then need to find it
-        if self.current_has_empty_dimensions() {
+        if self.current.layout.dimensions.width == 0.0
+            || self.current.layout.dimensions.height == 0.0
+        {
             self.current = Box::new(std::mem::take(&mut self.next));
 
             self.compositors
                 .advanced
                 .calculate_dimensions(&self.current);
 
+            self.compositors.advanced.update_layout(&self.current);
+
             self.compositors.elementary.set_should_resize();
             self.reset_next();
             self.latest_change = SugarTreeDiff::LayoutIsDifferent;
-            log::info!("current_has_empty_dimensions, will try to find...");
+            log::info!("has empty dimensions, will try to find...");
             return;
         }
 
         let mut should_update = false;
-        let mut should_clean_blocks = false;
         let mut should_resize = false;
         let mut should_compute_dimensions = false;
 
         self.latest_change =
             self.current
                 .calculate_diff(&self.next, false, self.is_dirty);
+
         match &self.latest_change {
             SugarTreeDiff::Equal => {
                 // Do nothing
@@ -279,13 +253,8 @@ impl SugarState {
             SugarTreeDiff::LayoutIsDifferent => {
                 should_update = true;
                 should_compute_dimensions = true;
-                should_clean_blocks = true;
                 should_resize = true;
             }
-            SugarTreeDiff::BlocksAreDifferent => {
-                should_clean_blocks = true;
-                should_update = true;
-            }
             SugarTreeDiff::LineQuantity(_) => {
                 should_update = true;
                 should_compute_dimensions = true;
@@ -312,22 +281,12 @@ impl SugarState {
             self.compositors.advanced.update_layout(&self.current);
         }
 
-        if should_clean_blocks {
-            self.compositors.elementary.clean_blocks();
-        }
-
         if should_resize {
             self.compositors.elementary.set_should_resize();
         }
 
         self.reset_next();
     }
-
-    #[inline]
-    pub fn current_has_empty_dimensions(&self) -> bool {
-        self.current.layout.dimensions.width == 0.0
-            || self.current.layout.dimensions.height == 0.0
-    }
 }
 
 // TODO: Write tests for compute layout updates
