@@ -18,6 +18,7 @@ use std::io;
 use std::io::{Error, ErrorKind};
 use std::mem::MaybeUninit;
 use std::ops::Deref;
+use std::os::fd::OwnedFd;
 use std::os::fd::{AsRawFd, FromRawFd, RawFd};
 use std::os::unix::process::CommandExt;
 use std::path::PathBuf;
@@ -493,9 +494,11 @@ pub fn create_pty_with_spawn(
     // Ownership of fd is transferred to the Stdio structs and will be closed by them at the end of
     // this scope. (It is not an issue that the fd is closed three times since File::drop ignores
     // error on libc::close.).
-    builder.stdin(unsafe { Stdio::from_raw_fd(child) });
-    builder.stderr(unsafe { Stdio::from_raw_fd(child) });
-    builder.stdout(unsafe { Stdio::from_raw_fd(child) });
+    let owned_child = unsafe { OwnedFd::from_raw_fd(child) };
+
+    builder.stdin(owned_child.try_clone()?);
+    builder.stderr(owned_child.try_clone()?);
+    builder.stdout(owned_child);
 
     builder.env("USER", user.user);
     builder.env("HOME", user.home);
